/*
KrossWordPuzzle - A crossword playing game and editor for KDE4.
This file contains code for a widget that allows editing of crossword answers.
Copyright (C) 2009     Friedrich Pülz <fpuelz@gmx.de>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "answerwidget.h"
#include <KLocalizedString>
#include <QFocusEvent>
#include <QDebug>

AnswerWidget::AnswerWidget( QWidget* parent )
	: KLineEdit( parent ) {
  setToolTip( i18n("The correct answer to the clue. For each letter a letter "
		   "cell is created") );
  setClickMessage( i18n("The answer to the clue") );
  setClearButtonShown( true );
  m_justGotFocusByMouse = false;
}

AnswerWidget::AnswerWidget( const QString &text, QWidget *parent )
	: KLineEdit( text, parent ) {
  setToolTip( i18n("The correct answer to the clue. For each letter a letter "
		   "cell is created") );
  setClickMessage( i18n("The answer to the clue") );
  setClearButtonShown( true );
  m_justGotFocusByMouse = false;
}

void AnswerWidget::focusInEvent( QFocusEvent* ev ) {
  KLineEdit::focusInEvent( ev );

  if ( ev->reason() == Qt::MouseFocusReason )
    m_justGotFocusByMouse = true;
}

void AnswerWidget::mousePressEvent( QMouseEvent* ev ) {
  if ( m_justGotFocusByMouse ) {
    m_justGotFocusByMouse = false;
    if ( cursorPositionAt(ev->pos()) > text().length() )
      home( false );
    else
      KLineEdit::mousePressEvent( ev );
  } else
    KLineEdit::mousePressEvent( ev );
}
