/*
KrossWordPuzzle - A crossword playing game and editor for KDE4.
This file contains code for the crossword dictionary.
Copyright (C) 2009     Friedrich Pülz <fpuelz@gmx.de>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DICTIONARY_H
#define DICTIONARY_H

#include "ui_database_connection.h"

#include <QObject>
#include <QHash>
#include <QStringList>
#include <QThread>
#include <QSqlDatabase>

class QProgressBar;
class KDialog;
class ExtendedSqlTableModel;

class KrosswordDictionary : public QObject {
  Q_OBJECT

  public:
	KrosswordDictionary( QObject* parent = 0 );
	virtual ~KrosswordDictionary();

	static const int MAX_WORD_LENGTH = 256;
	bool openDatabase( QWidget *dlgParent );
	void closeDatabase();
	bool createTables();

	bool isDatabaseOk() const { return m_databaseOk; };

	bool isEmpty();
	int entryCount();

	bool exportToCsv( const QString &fileName );
	int importFromCsv( const QString &fileName, QWidget *parent );

	ExtendedSqlTableModel *createModel();

	int addEntriesFromCrosswords( const QStringList &fileNames, QWidget *parent );
	int addEntriesFromDictionary( const QString &fileName, QWidget *parent );

	bool clearDatabase();

    public slots:
	void cancelCurrentActionClicked();

    signals:
	void extractedEntriesFromCrossword( const QString &fileName, int entryCount );
	void errorExtractedEntriesFromCrossword( const QString &fileName, const QString &errorString );

    private:
	KDialog *createProgressDialog( QWidget *parent, const QString &text, QProgressBar *progressBar );
	bool checkDatabase();

	Ui::database_connection ui_database_connection;
	bool m_cancel;
	bool m_databaseOk;
};

#endif // DICTIONARY_H
