/*
    KrossWordPuzzle - A crossword playing game and editor for KDE4.
    This file contains code for writing crosswords to XML files.
    Copyright (C) 2009     Friedrich P?lz <fpuelz@gmx.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef KROSSWORDWRITER_H
#define KROSSWORDWRITER_H

#include <QXmlStreamWriter>
#include <krossword.h>

namespace Crossword {
  class ImageCell;
  class SolutionLetterCell;
  class ClueCell;
  class KrossWord;
}
using namespace Crossword;

class KrossWordXmlWriter : public QXmlStreamWriter {
  public:
    KrossWordXmlWriter() { };

    bool writeCompressed( QIODevice *device, KrossWord *krossWord,
			  KrossWord::WriteMode writeMode = KrossWord::Normal,
			  const QByteArray &undoData = QByteArray() );
    bool write( QIODevice *device, KrossWord *krossWord,
		KrossWord::WriteMode writeMode = KrossWord::Normal,
		const QByteArray &undoData = QByteArray() );

    QString errorString() const { return m_errorString; };
		
  private:
    void writeKrossWord( KrossWord *krossWord,
			 KrossWord::WriteMode writeMode = KrossWord::Normal,
			 const QByteArray &undoData = QByteArray() );
    void writeClue( ClueCell *clue,
		    KrossWord::WriteMode writeMode = KrossWord::Normal );
    void writeImage( ImageCell *image,
		     KrossWord::WriteMode writeMode = KrossWord::Normal );
    void writeSolutionLetter( SolutionLetterCell *solutionLetter );

    QString m_errorString;
};

#endif // KROSSWORDWRITER_H