/*
    KrossWordPuzzle - A crossword playing game and editor for KDE4.
    This file contains code for the main window.
    Copyright (C) 2009     Friedrich P?lz <fpuelz@gmx.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef KROSSWORDPUZZLE_H
#define KROSSWORDPUZZLE_H

#include <kxmlguiwindow.h>

#include "ui_start_new.h"
#include "ui_settings.h"
// #include "ui_print_crossword.h"

#include "krossword.h"
#include "crosswordxmlguiwindow.h"
#include "krosswordtheme.h"

namespace KIO
{
    class PreviewJob;
}
class LibraryXmlGuiWindow;
class QTreeView;
class KRecentFilesAction;
class KUndoStack;
class KUrl;

/**
 * This class serves as the main window for KrossWordPuzzle.  It handles the
 * menus, toolbars, and status bars.
 *
 * @short Main window class
 * @author Friedrich P?lz <fpuelz@gmx.de>
 * @version %{VERSION}
 */
class KrossWordPuzzle : public KXmlGuiWindow
{
    Q_OBJECT

public:
    enum Action {
	Game_Download,
	Game_Upload,

    // 	Options_Themes,

	RecentTab_RecentFilesRemove,
    };

    const char *actionName( Action action ) const;

    /**
     * Default Constructor
     */
    KrossWordPuzzle();

    /**
     * Default Destructor
     */
    virtual ~KrossWordPuzzle();

    CrossWordXmlGuiWindow *mainCrossword() const { return m_mainCrossword; };

    void loadFile( const KUrl &url,
		   Crossword::KrossWord::FileFormat fileFormat = Crossword::KrossWord::DetermineByFileName,
		   bool loadCrashedFile = false );
    bool createNewCrossWord( const Crossword::CrosswordTypeInfo &crosswordTypeInfo,
			     const QSize &crosswordSize, const QString &title,
			     const QString &authors, const QString &copyright,
			     const QString &notes );
    bool createNewCrossWordFromTemplate( const QString &templateFilePath,
					 const QString &title,
					 const QString &authors,
					 const QString &copyright,
					 const QString &notes );
    /** Removes the path for crosswords that are in the library. */
    QString displayFileName( const QString &fileName );
    /** Checks if the crossword with the given filename is in the library. */
    bool isFileInLibrary( const QString &fileName );

protected:
    virtual void closeEvent( QCloseEvent *event );
    virtual void dropEvent( QDropEvent* );
    virtual void dragEnterEvent( QDragEnterEvent* );

public slots:
    // Game actions
    void gameNewSlot();
    void downloadSlot();
    void uploadSlot();
    void loadSlot( const KUrl &fileName = KUrl() );
    void loadRecentSlot( const KUrl &url );
    void loadFile( const QString &fileName );

    void saveSlot();
    void saveAsSlot();

    // Settings actions
    void optionsPreferencesSlot();
    void settingsChanged();
    AnimationTypes animationTypesFromSettings();
//     void changeThemeSlot( int themeId );

    void showStatusbarGlobal( bool show );
    int configureShortcutsGlobal();
    void configureToolbarsGlobal();
    void currentTabChanged( int );

protected slots:
    void crosswordLoadingComplete( const QString &fileName );
    void crosswordErrorLoading( const QString &fileName );
    void crosswordCurrentChanged( const QString &fileName, const QString &oldFileName );
    void crosswordClosed( const QString &fileName );
    void crosswordModificationsChanged( CrossWordXmlGuiWindow::ModificationTypes modificationTypes );
    void crosswordAutoSaveFileChanged( const QString &fileName );

//     void loadRecentItem();
//     void recentFileExecuted( QListWidgetItem *item );
//     void recentFileListContextMenuRequested( const QPoint &pos );

    // Popup menu actions
//     void recentFilesRemoveSlot();
//     void recentFilesClearSlot();

private:
    KDialog *createLoadProgressDialog();
    void setupPlaces();
    void setupActions();

    QMenu *popupMenuRecentFilesList();

    QWidget *createStartNewPage();


    Ui::start_new ui_start_new;
    Ui::settings ui_settings;

    QWidget *m_startNewPage;
    KMenuBar *m_cornerMenu;
    LibraryXmlGuiWindow *m_mainLibrary;
    CrossWordXmlGuiWindow *m_mainCrossword;
    KDialog *m_loadProgressDialog;

    KRecentFilesAction *m_recentFilesAction;
    QList<KToolBar*> m_hiddenToolBars;

    QString m_caption;
};

#endif // _KROSSWORDPUZZLE_H_
