/*
    KrossWordPuzzle - A crossword playing game and editor for KDE4.
    This file contains code for the crossword view, which is used as the
    central widget of the main window of the game to show a crossword.
    Copyright (C) 2009     Friedrich P?lz <fpuelz@gmx.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "krosswordpuzzleview.h"
#include "settings.h"

#include <klocale.h>
#include <QtGui/QLabel>
// #include <QGLWidget>
#include "krosswordrenderer.h"

KrossWordPuzzleView::KrossWordPuzzleView( KrossWordPuzzleScene *scene, QWidget *parent )
	    : QGraphicsView( scene, parent ), m_scene( scene ) {
//     setOptimizationFlags( QGraphicsView::DontSavePainterState );
    setRenderHints( QPainter::Antialiasing | QPainter::TextAntialiasing
		    | QPainter::SmoothPixmapTransform );

//     setMinimumSize( 100, 100 );
    setCacheMode( CacheBackground );
    settingsChanged();
//     setViewport( new QGLWidget );
//     setAutoFillBackground(true);

    setObjectName( "krosswordpuzzleview" );
}

KrossWordPuzzleView::~KrossWordPuzzleView()
{

}

QSize KrossWordPuzzleView::sizeHint() const
{
    if ( krossWord() ) {
	QSize sz = krossWord()->cellSize().toSize() * 0.8;
// 	kDebug() << QSize( krossWord()->width() * sz.width(), krossWord()->height() * sz.height() );
	return QSize( krossWord()->width() * sz.width() + 4, krossWord()->height() * sz.height() );
// 	return mapFromScene( krossWord()->boundingRect() ).boundingRect().size();
    } else
	return QGraphicsView::sizeHint();
}

void KrossWordPuzzleView::keyPressEvent( QKeyEvent* event )
{
    if ( event->modifiers().testFlag(Qt::ControlModifier) ) {
	krossWord()->setAcceptedMouseButtons( Qt::NoButton );
	setDragMode( QGraphicsView::ScrollHandDrag );
    }

    QGraphicsView::keyPressEvent( event );
}

void KrossWordPuzzleView::keyReleaseEvent( QKeyEvent* event )
{
    if ( !event->modifiers().testFlag(Qt::ControlModifier) ) {
	krossWord()->setAcceptedMouseButtons( Qt::LeftButton | Qt::MidButton | Qt::RightButton );
	setDragMode( QGraphicsView::NoDrag );
    }

    QGraphicsView::keyReleaseEvent( event );
}

void KrossWordPuzzleView::wheelEvent( QWheelEvent* event ) {
    if ( event->modifiers().testFlag(Qt::ControlModifier) )
      emit signalChangeZoom( event->delta() / 10 );
    else
      QGraphicsView::wheelEvent( event );
}

void KrossWordPuzzleView::resizeEvent( QResizeEvent* event )
{
    QGraphicsView::resizeEvent( event );
    emit resized( event->oldSize(), event->size() );
}
/*
void KrossWordPuzzleView::setTheme( const QString &theme )
{
    if ( !KrosswordRenderer::self()->setTheme(theme) )
	return;

    scene()->update();
    Settings::setTheme( theme );
    settingsChanged();
}*/

void KrossWordPuzzleView::renderToPrinter( QPainter* painter,
	const QRectF& target, const QRect& source, Qt::AspectRatioMode aspectRatioMode )
{
    bool wasDrawingForPrinting = krossWord()->isDrawingForPrinting();
    krossWord()->setDrawForPrinting();

    render( painter, target, source, aspectRatioMode );

    krossWord()->setDrawForPrinting( wasDrawingForPrinting );
}

void KrossWordPuzzleView::settingsChanged()
{
//     QPalette pal;
//     pal.setColor( QPalette::Window, Settings::col_background());
//     pal.setColor( QPalette::WindowText, Settings::col_foreground());
//     ui_krosswordpuzzleview_base.kcfg_sillyLabel->setPalette( pal );

//     ui_krosswordpuzzleview_base.kcfg_sillyLabel->setText( i18n("This project is %1 days old",Settings::val_time()) );
    Settings::self()->writeConfig();
    emit signalChangeStatusbar( i18n("Settings changed") );
}

#include "krosswordpuzzleview.moc"
