/*
    KrossWordPuzzle - A crossword playing game and editor for KDE4.
    This file contains code for the crossword view, which is used as the
    central widget of the main window of the game to show a crossword.
    Copyright (C) 2009     Friedrich P?lz <fpuelz@gmx.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef KROSSWORDPUZZLEVIEW_H
#define KROSSWORDPUZZLEVIEW_H

// #include <QtGui/QWidget>
#include <QGraphicsView>

// #include "ui_krosswordpuzzleview_base.h"
#include "krosswordpuzzlescene.h"

// class QGraphicsView;
class QPainter;
class KUrl;

/**
 * This is the main view class for KrossWordPuzzle.
 *
 * @short Main view
 * @author Friedrich P?lz <fpuelz@gmx.de>
 * @version 0.15
 */

class KrossWordPuzzleView : public QGraphicsView //, public Ui::krosswordpuzzleview_base
{
    Q_OBJECT
public:
    /**
     * Default constructor
     */
    KrossWordPuzzleView( KrossWordPuzzleScene *scene, QWidget *parent = 0 );

    /**
     * Destructor
     */
    virtual ~KrossWordPuzzleView();

    inline Crossword::KrossWord *krossWord() const {
	return ((KrossWordPuzzleScene*)scene())->krossWord(); };
//     void setTheme( const QString &theme = "default" );

    void renderToPrinter( QPainter *painter, const QRectF &target = QRectF(),
		const QRect &source = QRect(),
                Qt::AspectRatioMode aspectRatioMode = Qt::KeepAspectRatio );

    virtual QSize sizeHint() const;

protected:
    virtual void keyPressEvent( QKeyEvent* event );
    virtual void keyReleaseEvent( QKeyEvent* event );
    virtual void wheelEvent( QWheelEvent* event );

    virtual void resizeEvent( QResizeEvent* event );

private:
//     Ui::krosswordpuzzleview_base ui_krosswordpuzzleview_base;
    KrossWordPuzzleScene *m_scene;

signals:
    /**
     * Use this signal to change the content of the statusbar
     */
    void signalChangeStatusbar( const QString& text );

    /**
     * Use this signal to change the content of the caption
     */
    void signalChangeCaption( const QString& text );

    void signalChangeZoom( int zoomChange );

    void resized( const QSize &oldSize, const QSize &newSize );

private slots:
    void settingsChanged();
};

#endif // KrossWordPuzzleVIEW_H
